from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional, Dict, Any
import uuid


@dataclass
class Entry:
    content: str
    title: Optional[str] = None
    tags: List[str] = field(default_factory=list)
    id: str = field(default_factory=lambda: str(uuid.uuid4()))
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: Optional[datetime] = None
    # El diccionario de metadatos para flexibilidad total
    metadata: Dict[str, Any] = field(default_factory=dict)

    def update_content(self, new_content: str):
        self.content = new_content
        self.updated_at = datetime.now()



import pytest
from datetime import datetime
from dataclasses import dataclass, field
from typing import List
from datetime import timezone

from CapsuleCore_book.capsule.OrjsonKnowledgeRepository import OrjsonKnowledgeRepository


# --- Mocks de las clases core para que el test funcione ---
@dataclass
class Entry:
    id: str
    content: str
    tags: List[str] = field(default_factory=list)
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)


@dataclass
class Relation:
    from_id: str
    to_id: str
    type: str


# --- Fixtures ---


@pytest.fixture
def repo(tmp_path):
    """Crea una instancia del repositorio en un directorio temporal."""
    return OrjsonKnowledgeRepository(storage_path=str(tmp_path))


@pytest.fixture
def sample_entry():
    return Entry(
        id="test-123",
        content="Contenido de prueba",
        tags=["python", "test"],
        # 2. Añade el tzinfo aquí:
        created_at=datetime(2023, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
        updated_at=datetime(2023, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
    )


# --- Tests ---


def test_save_and_get_by_id(repo, sample_entry):
    # Guardar
    repo.save(sample_entry)

    # Recuperar
    retrieved = repo.get_by_id(sample_entry.id)

    assert retrieved is not None
    assert retrieved.id == sample_entry.id
    assert retrieved.content == sample_entry.content
    assert "python" in retrieved.tags
    # Verificar que la hidratación de fechas funciona
    assert isinstance(retrieved.created_at, datetime)
    assert retrieved.created_at == sample_entry.created_at


def test_get_non_existent_entry(repo):
    assert repo.get_by_id("ghost-id") is None


def test_delete_entry_and_file(repo, sample_entry):
    repo.save(sample_entry)
    file_path = repo._get_entry_path(sample_entry.id)
    assert file_path.exists()

    success = repo.delete(sample_entry.id)
    assert success is True
    assert not file_path.exists()
    assert repo.get_by_id(sample_entry.id) is None


def test_find_by_tag(repo):
    entry1 = Entry(id="1", content="A", tags=["IA"])
    entry2 = Entry(id="2", content="B", tags=["ia", "coding"])
    entry3 = Entry(id="3", content="C", tags=["linux"])

    repo.save(entry1)
    repo.save(entry2)
    repo.save(entry3)

    # Búsqueda case-insensitive y con espacios
    results = repo.find_by_tag(" ia ")

    assert len(results) == 2
    ids = [e.id for e in results]
    assert "1" in ids
    assert "2" in ids


def test_add_and_remove_relation(repo):
    rel = Relation(from_id="A", to_id="B", type="references")

    # Agregar
    repo.add_relation(rel)
    relations = repo._read_relations()
    assert len(relations) == 1
    assert relations[0]["from_id"] == "A"

    # Eliminar
    repo.remove_relation(rel)
    assert len(repo._read_relations()) == 0


def test_delete_entry_cleans_relations(repo, sample_entry):
    # Setup: Una entrada con una relación
    repo.save(sample_entry)
    rel = Relation(from_id=sample_entry.id, to_id="other", type="links")
    repo.add_relation(rel)

    assert len(repo._read_relations()) == 1

    # Al borrar la entrada, la relación debe desaparecer
    repo.delete(sample_entry.id)
    assert len(repo._read_relations()) == 0



from .CodexService import CodexService
from .KnowledgeRepository import KnowledgeRepository
from .SQLiteKnowledgeRepository import SQLiteKnowledgeRepository
from .OrjsonKnowledgeRepository import OrjsonKnowledgeRepository


__all__ = [
    "CodexService",
    "KnowledgeRepository",
    "SQLiteKnowledgeRepository",
    "OrjsonKnowledgeRepository",
]



from abc import ABC, abstractmethod

from ..core import Entry, Relation
from typing import List, Optional


class KnowledgeRepository(ABC):
    @abstractmethod
    def save(self, entry: Entry) -> None:
        pass

    @abstractmethod
    def get_by_id(self, entry_id: str) -> Optional[Entry]:
        pass

    @abstractmethod
    def find_by_tag(self, tag: str) -> List[Entry]:
        pass

    @abstractmethod
    def delete(self, entry_id: str) -> bool:
        """Elimina una entrada por su ID."""
        pass

    @abstractmethod
    def add_relation(self, relation: Relation) -> None:
        pass

    @abstractmethod
    def remove_relation(self, relation: Relation) -> None:
        pass



import pytest
from typing import List
from dataclasses import dataclass

from CapsuleCore_book.capsule.KnowledgeRepository import KnowledgeRepository


# --- Mocks de dependencias (ajusta según tus archivos reales) ---
@dataclass
class Entry:
    id: str
    content: str
    tags: List[str]


@dataclass
class Relation:
    from_id: str
    to_id: str


# --- La Suite de Tests Base ---
class KnowledgeRepositoryTests:
    """
    Clase base de pruebas. No se ejecuta sola, se hereda en
    las implementaciones concretas.
    """

    def test_save_and_get_by_id(self, repo):
        entry = Entry(id="1", content="Test", tags=["tech"])
        repo.save(entry)

        result = repo.get_by_id("1")
        assert result == entry
        assert result.id == "1"

    def test_get_by_id_returns_none_if_not_exists(self, repo):
        assert repo.get_by_id("non-existent") is None

    def test_find_by_tag(self, repo):
        e1 = Entry(id="1", content="A", tags=["python"])
        e2 = Entry(id="2", content="B", tags=["python", "ai"])
        e3 = Entry(id="3", content="C", tags=["ai"])

        for e in [e1, e2, e3]:
            repo.save(e)

        python_entries = repo.find_by_tag("python")
        assert len(python_entries) == 2
        assert e1 in python_entries
        assert e2 in python_entries

    def test_delete_entry(self, repo):
        entry = Entry(id="del-me", content="Bye", tags=[])
        repo.save(entry)

        deleted = repo.delete("del-me")
        assert deleted is True
        assert repo.get_by_id("del-me") is None

    def test_delete_non_existent_returns_false(self, repo):
        assert repo.delete("ghost") is False


# Implementación concreta simple para el ejemplo
class InMemoryRepository(KnowledgeRepository):
    def __init__(self):
        self.entries = {}
        self.relations = []

    def save(self, entry):
        self.entries[entry.id] = entry

    def get_by_id(self, entry_id):
        return self.entries.get(entry_id)

    def find_by_tag(self, tag):
        return [e for e in self.entries.values() if tag in e.tags]

    def delete(self, entry_id):
        return self.entries.pop(entry_id, None) is not None

    def add_relation(self, relation):
        self.relations.append(relation)

    def remove_relation(self, relation):
        self.relations.remove(relation)


# --- El Test Final ---
class TestInMemoryRepository(KnowledgeRepositoryTests):
    @pytest.fixture
    def repo(self):
        return InMemoryRepository()



import pytest
from dataclasses import FrozenInstanceError
from CapsuleCore_book.core.Relation import (
    Relation,
)  # Reemplaza 'tu_archivo' con el nombre real


def test_relation_creation():
    """Verifica que los atributos se asignen correctamente al instanciar."""
    rel = Relation(from_id="A", to_id="B", connection_type="referencia")

    assert rel.from_id == "A"
    assert rel.to_id == "B"
    assert rel.connection_type == "referencia"


def test_relation_immutability():
    """Verifica que no se puedan modificar los atributos (frozen=True)."""
    rel = Relation(from_id="A", to_id="B", connection_type="depende de")

    with pytest.raises(FrozenInstanceError):
        rel.from_id = "C"


def test_relation_equality():
    """Verifica que dos instancias con los mismos datos sean iguales."""
    rel1 = Relation("1", "2", "tipo")
    rel2 = Relation("1", "2", "tipo")

    assert rel1 == rel2


def test_relation_hashable():
    """Verifica que, al ser frozen, se puede usar en sets o como llaves de dict."""
    rel = Relation("A", "B", "vínculo")
    mi_set = {rel}

    assert rel in mi_set



import pytest
import sqlite3
from datetime import datetime
from typing import List, Optional
from dataclasses import dataclass, field

from CapsuleCore_book.capsule.SQLiteKnowledgeRepository import SQLiteKnowledgeRepository


# --- Importa aquí tus clases reales ---
# from tu_modulo import SQLiteKnowledgeRepository, Entry, Relation


@dataclass
class Entry:
    id: str
    title: str
    content: str
    tags: List[str]
    created_at: datetime
    updated_at: Optional[datetime] = None
    metadata: dict = field(default_factory=dict)


@dataclass
class Relation:
    from_id: str
    to_id: str
    connection_type: str


# --- Fixtures ---


@pytest.fixture
def temp_db(tmp_path):
    """Crea una ruta temporal para la base de datos en cada test."""
    db_file = tmp_path / "test_knowledge.db"
    return str(db_file)


@pytest.fixture
def repo(temp_db):
    """Instancia el repositorio usando el archivo temporal."""
    return SQLiteKnowledgeRepository(db_path=temp_db)


@pytest.fixture
def sample_entry():
    return Entry(
        id="entry-123",
        title="Testing Pytest",
        content="Contenido de prueba para el repositorio",
        tags=["unit-test", "python"],
        created_at=datetime(2024, 5, 20, 10, 30),
        updated_at=datetime(2024, 5, 20, 11, 0),
        metadata={"priority": "high"},
    )


# --- Tests ---


class TestSQLiteKnowledgeRepository:
    def test_save_and_retrieve(self, repo, sample_entry):
        # 1. Guardar
        repo.save(sample_entry)

        # 2. Recuperar y verificar campos
        retrieved = repo.get_by_id(sample_entry.id)

        assert retrieved is not None
        assert retrieved.id == "entry-123"
        assert retrieved.title == "Testing Pytest"
        assert "unit-test" in retrieved.tags
        assert retrieved.metadata["priority"] == "high"
        assert isinstance(retrieved.created_at, datetime)

    def test_upsert_behavior(self, repo, sample_entry):
        repo.save(sample_entry)

        # Modificar solo el título
        sample_entry.title = "Nuevo Título"
        repo.save(sample_entry)

        retrieved = repo.get_by_id(sample_entry.id)
        assert retrieved.title == "Nuevo Título"
        assert retrieved.content == "Contenido de prueba para el repositorio"

    def test_find_by_tag_search(self, repo, sample_entry):
        repo.save(sample_entry)

        # Añadir otra entrada con tags distintos
        entry2 = Entry(
            id="entry-456",
            title="Otro",
            content="...",
            tags=["sql"],
            created_at=datetime.now(),
        )
        repo.save(entry2)

        # Buscar tag que existe en sample_entry
        results = repo.find_by_tag("python")
        assert len(results) == 1
        assert results[0].id == "entry-123"

    def test_delete_cascades_relations(self, repo, sample_entry):
        # Crear dos entradas
        entry2 = Entry(
            id="entry-2", title="B", content="...", tags=[], created_at=datetime.now()
        )
        repo.save(sample_entry)
        repo.save(entry2)

        # Crear relación
        rel = Relation(
            from_id=sample_entry.id, to_id=entry2.id, connection_type="links"
        )
        repo.add_relation(rel)

        # Borrar la entrada origen
        repo.delete(sample_entry.id)

        # Verificar que la entrada no existe
        assert repo.get_by_id(sample_entry.id) is None

        # Verificar que la relación se limpió (vía manual si no tienes get_relations)
        with sqlite3.connect(repo.db_path) as conn:
            res = conn.execute(
                "SELECT * FROM relations WHERE from_id = ?", (sample_entry.id,)
            ).fetchall()
            assert len(res) == 0

    def test_get_non_existent_returns_none(self, repo):
        assert repo.get_by_id("non-existent") is None

    def test_remove_specific_relation(self, repo):
        # Setup de dos entradas y una relación
        e1 = Entry(id="a", title="T1", content="C1", tags=[], created_at=datetime.now())
        e2 = Entry(id="b", title="T2", content="C2", tags=[], created_at=datetime.now())
        repo.save(e1)
        repo.save(e2)
        rel = Relation(from_id="a", to_id="b", connection_type="test")

        repo.add_relation(rel)
        repo.remove_relation(rel)

        with sqlite3.connect(repo.db_path) as conn:
            res = conn.execute("SELECT * FROM relations WHERE from_id = 'a'").fetchall()
            assert len(res) == 0



import sqlite3
import json
from datetime import datetime
from typing import List, Optional
from ..core import Entry, Relation
from .KnowledgeRepository import KnowledgeRepository


class SQLiteKnowledgeRepository(KnowledgeRepository):
    def __init__(self, db_path: str = "codex.db"):
        self.db_path = db_path
        self._create_tables()

    def _get_connection(self):
        conn = sqlite3.connect(self.db_path)
        conn.row_factory = sqlite3.Row
        return conn

    def _create_tables(self):
        with self._get_connection() as conn:
            # Tabla de Entradas
            conn.execute("""
                CREATE TABLE IF NOT EXISTS entries (
                    id TEXT PRIMARY KEY,
                    title TEXT,
                    content TEXT,
                    tags TEXT,
                    created_at TEXT,
                    updated_at TEXT,
                    metadata TEXT
                )
            """)
            # Tabla de Relaciones (con llaves foráneas opcionales para integridad)
            conn.execute("""
                CREATE TABLE IF NOT EXISTS relations (
                    from_id TEXT,
                    to_id TEXT,
                    connection_type TEXT,
                    PRIMARY KEY (from_id, to_id, connection_type),
                    FOREIGN KEY (from_id) REFERENCES entries (id) ON DELETE CASCADE,
                    FOREIGN KEY (to_id) REFERENCES entries (id) ON DELETE CASCADE
                )
            """)

    def save(self, entry: Entry) -> None:
        """Upsert: Crea o actualiza la entrada completa."""
        with self._get_connection() as conn:
            conn.execute(
                """
                INSERT INTO entries (id, title, content, tags, created_at, updated_at, metadata)
                VALUES (?, ?, ?, ?, ?, ?, ?)
                ON CONFLICT(id) DO UPDATE SET
                    title=excluded.title,
                    content=excluded.content,
                    tags=excluded.tags,
                    updated_at=excluded.updated_at,
                    metadata=excluded.metadata
            """,
                (
                    entry.id,
                    entry.title,
                    entry.content,
                    json.dumps(entry.tags),
                    entry.created_at.isoformat(),
                    entry.updated_at.isoformat() if entry.updated_at else None,
                    json.dumps(entry.metadata),
                ),
            )

    def delete(self, entry_id: str) -> bool:
        """Elimina una entrada y sus relaciones asociadas."""
        with self._get_connection() as conn:
            cursor = conn.execute("DELETE FROM entries WHERE id = ?", (entry_id,))
            # También limpiamos las relaciones donde participaba esta entrada
            conn.execute(
                "DELETE FROM relations WHERE from_id = ? OR to_id = ?",
                (entry_id, entry_id),
            )
            return cursor.rowcount > 0

    def get_by_id(self, entry_id: str) -> Optional[Entry]:
        with self._get_connection() as conn:
            row = conn.execute(
                "SELECT * FROM entries WHERE id = ?", (entry_id,)
            ).fetchone()
            if not row:
                return None

            return Entry(
                id=row["id"],
                title=row["title"],
                content=row["content"],
                tags=json.loads(row["tags"]),
                created_at=datetime.fromisoformat(row["created_at"]),
                updated_at=datetime.fromisoformat(row["updated_at"])
                if row["updated_at"]
                else None,
                metadata=json.loads(row["metadata"]),
            )

    def find_by_tag(self, tag: str) -> List[Entry]:
        """Busca entradas que contengan el tag específico en su JSON."""
        with self._get_connection() as conn:
            # Usamos LIKE para buscar dentro del string JSON de tags
            search_pattern = f'%"{tag.lower().strip()}"%'
            rows = conn.execute(
                "SELECT * FROM entries WHERE tags LIKE ?", (search_pattern,)
            ).fetchall()

            entries = []
            for row in rows:
                entries.append(
                    Entry(
                        id=row["id"],
                        title=row["title"],
                        content=row["content"],
                        tags=json.loads(row["tags"]),
                        created_at=datetime.fromisoformat(row["created_at"]),
                        updated_at=datetime.fromisoformat(row["updated_at"])
                        if row["updated_at"]
                        else None,
                        metadata=json.loads(row["metadata"]),
                    )
                )
            return entries

    def add_relation(self, relation: Relation) -> None:
        with self._get_connection() as conn:
            conn.execute(
                "INSERT OR IGNORE INTO relations (from_id, to_id, connection_type) VALUES (?, ?, ?)",
                (relation.from_id, relation.to_id, relation.connection_type),
            )

    def remove_relation(self, relation: Relation) -> None:
        """Elimina una conexión específica entre dos entradas."""
        with self._get_connection() as conn:
            conn.execute(
                "DELETE FROM relations WHERE from_id = ? AND to_id = ? AND connection_type = ?",
                (relation.from_id, relation.to_id, relation.connection_type),
            )



from datetime import datetime
from CapsuleCore_book.core.Entry import Entry  # Asegúrate de importar desde tu archivo


def test_entry_creation_defaults():
    """Valida que una entrada se cree correctamente con valores por defecto."""
    content = "Hola Mundo"
    entry = Entry(content=content)

    assert entry.content == content
    assert entry.title is None
    assert isinstance(entry.id, str)
    assert isinstance(entry.created_at, datetime)
    assert entry.tags == []
    assert entry.metadata == {}
    assert entry.updated_at is None


def test_entry_custom_values():
    """Valida la asignación de valores personalizados."""
    custom_id = "123-abc"
    entry = Entry(
        content="Contenido",
        title="Mi Título",
        tags=["python", "test"],
        id=custom_id,
        metadata={"author": "AI"},
    )

    assert entry.id == custom_id
    assert "python" in entry.tags
    assert entry.metadata["author"] == "AI"


def test_update_content_logic():
    """Verifica que el contenido cambie y se registre la fecha de actualización."""
    entry = Entry(content="Original")
    assert entry.updated_at is None

    original_created_at = entry.created_at
    new_text = "Editado"

    entry.update_content(new_text)

    assert entry.content == new_text
    assert isinstance(entry.updated_at, datetime)
    # El tiempo de creación no debería cambiar
    assert entry.created_at == original_created_at


def test_unique_ids():
    """Asegura que cada instancia genere un ID único."""
    entry1 = Entry(content="E1")
    entry2 = Entry(content="E2")
    assert entry1.id != entry2.id



from typing import List

from ..core import Entry, Relation
from .KnowledgeRepository import KnowledgeRepository


class CodexService:
    def __init__(self, repository: KnowledgeRepository):
        self.repository = repository

    def ingest(self, entry: Entry):
        # REGLA 1: Normalización de etiquetas
        entry = self._normalize_tags(entry)

        # REGLA 2: Evitar duplicados
        if self.repository.get_by_id(entry.id):
            raise ValueError("Esta página ya existe en el códice.")

        self.repository.save(entry)

    def _normalize_tags(self, entry: Entry):
        """Regla interna: Limpieza de etiquetas."""
        entry.tags = list(set(t.lower().strip() for t in entry.tags))
        return entry

    def create_page(self, title: str, content: str, tags: List[str] = None) -> Entry:
        """Caso de Uso: Crear un nuevo conocimiento con reglas de validación."""
        if not title.strip():
            raise ValueError("El título no puede estar vacío.")

        new_entry = Entry(title=title, content=content, tags=tags or [])

        return self.ingest(new_entry)

    def edit_page(self, entry_id: str, new_content: str):
        """Caso de Uso: Editar contenido y actualizar metadatos de auditoría."""
        entry = self.repository.get_by_id(entry_id)
        if not entry:
            raise ValueError(f"No existe la página con ID {entry_id}")

        # 1. Aplicamos el cambio al Core
        entry.update_content(new_content)

        # 2. Lógica de Metadatos: Contador de ediciones
        edits = entry.metadata.get("edit_count", 0)
        entry.metadata["edit_count"] = edits + 1

        # 3. Persistimos
        self.repository.save(entry)

    def connect_pages(self, from_id: str, to_id: str, connection_type: str):
        """Caso de Uso: Crear una relación validando la existencia de ambos nodos."""
        if from_id == to_id:
            raise ValueError("No puedes relacionar una página consigo misma.")

        # Validamos que ambos existan antes de crear la relación
        origin = self.repository.get_by_id(from_id)
        target = self.repository.get_by_id(to_id)

        if not origin or not target:
            raise ValueError("Ambas páginas deben existir para ser relacionadas.")

        relation = Relation(
            from_id=from_id, to_id=to_id, connection_type=connection_type
        )
        self.repository.add_relation(relation)

    def delete_page(self, entry_id: str):
        """Elimina una página y limpia sus conexiones para mantener la integridad."""
        if not self.repository.get_by_id(entry_id):
            raise ValueError(f"No se puede eliminar: la página {entry_id} no existe.")

        # 1. Limpiar relaciones (Dependiendo de tu repo, esto podría ser automático)
        self.repository.remove_all_relations_to(entry_id)

        # 2. Borrar la entidad
        self.repository.delete(entry_id)

    def disconnect_pages(self, from_id: str, to_id: str, connection_type: str = None):
        """Rompe el vínculo entre dos páginas."""
        # Podrías filtrar por connection_type si permites múltiples tipos entre nodos
        relation_exists = self.repository.check_relation(
            from_id, to_id, connection_type
        )

        if not relation_exists:
            raise ValueError("No existe una relación previa entre estas páginas.")

        self.repository.remove_relation(from_id, to_id, connection_type)



import pytest
from unittest.mock import MagicMock
from dataclasses import dataclass, field
from typing import List, Dict

from CapsuleCore_book.capsule.CodexService import CodexService


# --- Mocks de las entidades (basado en tu código) ---
@dataclass
class Entry:
    title: str
    content: str
    id: str = "123"
    tags: List[str] = field(default_factory=list)
    metadata: Dict = field(default_factory=dict)

    def update_content(self, new_content):
        self.content = new_content


@dataclass
class Relation:
    from_id: str
    to_id: str
    connection_type: str


# --- Tests para CodexService ---


class TestCodexService:
    @pytest.fixture
    def mock_repo(self):
        """Fixture para crear un repositorio simulado."""
        return MagicMock()

    @pytest.fixture
    def service(self, mock_repo):
        """Fixture para instanciar el servicio con el mock."""
        return CodexService(mock_repo)

    ## Tests de Ingesta y Creación
    ## --------------------------

    def test_create_page_success(self, service, mock_repo):
        # 1. Configuración
        mock_repo.get_by_id.return_value = None

        # 2. Ejecución (Usando AI y ai para forzar el duplicado)
        service.create_page("Título Válido", "Contenido", [" AI ", "ai", "Python"])

        # 3. Verificación
        assert mock_repo.save.called
        saved_entry = mock_repo.save.call_args[0][0]

        # Ahora sí coinciden los datos:
        assert len(saved_entry.tags) == 2
        assert "ai" in saved_entry.tags  # <-- Cambiado de "ia" a "ai"
        assert "python" in saved_entry.tags

    def test_ingest_duplicate_error(self, service, mock_repo):
        # Configuración: el repo ya tiene ese ID
        mock_repo.get_by_id.return_value = Entry("Existente", "...", id="1")

        entry = Entry("Nuevo", "...", id="1")
        with pytest.raises(ValueError, match="Esta página ya existe"):
            service.ingest(entry)

    ## Tests de Edición
    ## ----------------

    def test_edit_page_updates_metadata(self, service, mock_repo):
        # Configuración: existe una entrada previa
        existing_entry = Entry(
            "Orig", "Contenido viejo", id="abc", metadata={"edit_count": 1}
        )
        mock_repo.get_by_id.return_value = existing_entry

        service.edit_page("abc", "Nuevo contenido")

        assert existing_entry.content == "Nuevo contenido"
        assert existing_entry.metadata["edit_count"] == 2
        mock_repo.save.assert_called_with(existing_entry)

    ## Tests de Relaciones
    ## -------------------

    def test_connect_pages_same_id_error(self, service):
        with pytest.raises(ValueError, match="consigo misma"):
            service.connect_pages("A", "A", "link")

    def test_connect_pages_missing_target_error(self, service, mock_repo):
        # Solo existe el origen, no el destino
        mock_repo.get_by_id.side_effect = [Entry("A", ""), None]

        with pytest.raises(ValueError, match="Ambas páginas deben existir"):
            service.connect_pages("id_1", "id_2", "ref")

    ## Tests de Eliminación y Desconexión
    ## ----------------------------------

    def test_delete_page_cleans_relations(self, service, mock_repo):
        mock_repo.get_by_id.return_value = Entry("A", "")

        service.delete_page("A")

        mock_repo.remove_all_relations_to.assert_called_once_with("A")
        mock_repo.delete.assert_called_once_with("A")

    def test_disconnect_non_existent_relation_error(self, service, mock_repo):
        mock_repo.check_relation.return_value = False

        with pytest.raises(ValueError, match="No existe una relación previa"):
            service.disconnect_pages("A", "B")



import orjson
from datetime import datetime
from typing import List, Optional
from pathlib import Path
from dataclasses import asdict

from .KnowledgeRepository import KnowledgeRepository
from ..core import Entry, Relation


class OrjsonKnowledgeRepository(KnowledgeRepository):
    def __init__(self, storage_path: str = "./data/codex"):
        self.storage_path = Path(storage_path)
        self.relations_path = self.storage_path / "relations.json"

        # Crear directorios de persistencia si no existen
        self.storage_path.mkdir(parents=True, exist_ok=True)
        if not self.relations_path.exists():
            self._write_relations([])

    def _get_entry_path(self, entry_id: str) -> Path:
        return self.storage_path / f"{entry_id}.json"

    # --- Implementación del Contrato ---

    def save(self, entry: Entry) -> None:
        file_path = self._get_entry_path(entry.id)
        # OPT_SERIALIZE_DATACLASS: permite volcar la dataclass directamente
        # OPT_NAIVE_UTC: asegura que los datetime se guarden en formato ISO compatible
        data = orjson.dumps(
            entry,
            option=orjson.OPT_INDENT_2
            | orjson.OPT_SERIALIZE_DATACLASS
            | orjson.OPT_NAIVE_UTC,
        )
        with open(file_path, "wb") as f:
            f.write(data)

    def get_by_id(self, entry_id: str) -> Optional[Entry]:
        file_path = self._get_entry_path(entry_id)
        if not file_path.exists():
            return None

        with open(file_path, "rb") as f:
            raw_data = orjson.loads(f.read())

            # Re-hidratación de fechas (orjson carga fechas como strings ISO)
            if raw_data.get("created_at"):
                raw_data["created_at"] = datetime.fromisoformat(raw_data["created_at"])
            if raw_data.get("updated_at"):
                raw_data["updated_at"] = datetime.fromisoformat(raw_data["updated_at"])

            return Entry(**raw_data)

    def delete(self, entry_id: str) -> bool:
        """Elimina la entrada y limpia sus relaciones."""
        file_path = self._get_entry_path(entry_id)
        if not file_path.exists():
            return False

        # 1. Eliminar archivo de la entrada
        file_path.unlink()

        # 2. Limpiar relaciones donde el ID sea origen o destino
        relations = self._read_relations()
        filtered_relations = [
            r for r in relations if r["from_id"] != entry_id and r["to_id"] != entry_id
        ]

        if len(relations) != len(filtered_relations):
            self._write_relations(filtered_relations)

        return True

    def find_by_tag(self, tag: str) -> List[Entry]:
        results = []
        target_tag = tag.lower().strip()

        for file in self.storage_path.glob("*.json"):
            if file.name == "relations.json":
                continue

            entry = self.get_by_id(file.stem)
            if entry and any(target_tag == t.lower() for t in entry.tags):
                results.append(entry)
        return results

    # --- Gestión de Relaciones ---

    def _read_relations(self) -> List[dict]:
        try:
            with open(self.relations_path, "rb") as f:
                return orjson.loads(f.read())
        except orjson.JSONDecodeError, FileNotFoundError:
            return []

    def _write_relations(self, relations: List[dict]):
        with open(self.relations_path, "wb") as f:
            f.write(orjson.dumps(relations, option=orjson.OPT_INDENT_2))

    def add_relation(self, relation: Relation) -> None:
        relations = self._read_relations()
        rel_dict = asdict(relation)

        # Evitar duplicados exactos en la lista de relaciones
        if rel_dict not in relations:
            relations.append(rel_dict)
            self._write_relations(relations)

    def remove_relation(self, relation: Relation) -> None:
        relations = self._read_relations()
        rel_dict = asdict(relation)

        # Filtramos para quitar la relación específica
        new_relations = [r for r in relations if r != rel_dict]

        if len(relations) != len(new_relations):
            self._write_relations(new_relations)



from .core import Entry, Relation
from .capsule import (
    CodexService,
    KnowledgeRepository,
    SQLiteKnowledgeRepository,
    OrjsonKnowledgeRepository,
)


__all__ = [
    "Entry",
    "Relation",
    "CodexService",
    "KnowledgeRepository",
    "SQLiteKnowledgeRepository",
    "OrjsonKnowledgeRepository",
]
